# url link to JSON file
# file_name to cash the dataset if not exists
import datetime
import json
import math
import os
import sys
import urllib.request
from pathlib import Path
from time import sleep
from colorama import Fore
import pandas as pd
from tqdm import tqdm

import setify

pd.set_option('display.max_colwidth', None)

def load_data(origin, fname):
    return get_file(origin, fname)

def get_file(origin, fname, cache_dir=None, cache_subdir='datasets'):
    if cache_dir is None:
        cache_dir = os.path.join(str(Path.home()), '.setify')

    datadir_base = os.path.expanduser(cache_dir)
    if not os.access(str(Path.home()), os.W_OK):
        datadir_base = os.path.join('/tmp', '.setify')
    datadir = os.path.join(datadir_base, cache_subdir)

    if not os.path.exists(datadir):
        os.makedirs(datadir)
    fpath = os.path.join(datadir, fname)

    download = not os.path.exists(fpath)

    if download:
        print('Attempting to download data from', origin)
        try:
            with urllib.request.urlopen(origin) as response, pd.HDFStore(fpath, 'w') as f:
                content_type = response.info().get('Content-Length')
                total_size = int(content_type) if content_type is not None else -1

                chunk_size = max(int(total_size / 100), 1024 * 1024)  # At least 1MB
                print(Fore.WHITE + 'File size: ', convert_size(total_size))
                print(Fore.WHITE + 'File location: ', fpath)

                with tqdm(total=total_size, unit='B', unit_scale=True, ncols=100,
                          bar_format="{l_bar}%s{bar}%s{r_bar}" % (Fore.CYAN, Fore.RESET)) as progress:
                    data = ''
                    while True:
                        chunk = response.read(chunk_size)
                        if not chunk:
                            break
                        data += chunk.decode()
                        progress.update(len(chunk))

                json_data = json.loads(data)
                df = pd.DataFrame(json_data)
                f.append(key='/dataset1', value=df)
                f.get_storer('/dataset1').attrs.metadata = {'date-time': datetime.datetime.now(),
                                                           'version': setify.__version__}
        except Exception as e:
            print(Fore.RED + "Failed to download the file. Error: {}".format(e))
            return None

    return fpath


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])


def frame_information(df):
    x = df[:5]
    y = pd.DataFrame([['...'] * df.shape[1]],
                     columns=df.columns, index=['...'])
    z = df[-5:]
    frame = [x, y, z]
    result = pd.concat(frame)
    print(Fore.GREEN +  str(result))
    print('[' + str(df.shape[0]) + ' rows x ' + str(df.shape[1]) + ' columns]')
  
